// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef LIBKTORRENT_VERSION_H
#define LIBKTORRENT_VERSION_H

#define LIBKTORRENT_VERSION_STRING "25.12.3"
#define LIBKTORRENT_VERSION_MAJOR 25
#define LIBKTORRENT_VERSION_MINOR 12
#define LIBKTORRENT_VERSION_PATCH 3
#define LIBKTORRENT_VERSION ((25<<16)|(12<<8)|(3))

#endif
